/* 

                          Firewall Builder

                 Copyright (C) 2000 Vadim Kurland

  Author:  Vadim Kurland     vadim@vk.crocodile.org

  $Id: FWObjectDatabaseGUI.cc,v 1.8 2001/10/05 02:20:38 vkurland Exp $


  This program is free software which we release under the GNU General Public
  License. You may redistribute and/or modify this program under the terms
  of that license as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  To get a copy of the GNU General Public License, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
#include <time.h>

#include <config.h>

#include "FWObject.hh"
#include "FWObjectDatabaseGUI.hh"

#include "FWOptions.hh"

#include "Rule.hh"
#include "RuleElement.hh"
#include "Policy.hh"
#include "InterfacePolicy.hh"
#include "NAT.hh"
#include "ObjectGroup.hh"
#include "ServiceGroup.hh"
#include "IntervalGroup.hh"
#include "Host.hh"
#include "Firewall.hh"
#include "Network.hh"
#include "Interface.hh"

#include "FileSel.hh"
#include "MessageDialog.hh"
#include "main_window.hh"

using namespace libfwbuilder;

/****************************************************************/

Group*             FWObjectDatabaseGUI::ScratchPad = NULL;

const string FWObjectDatabaseGUI::DB_TEMPLATE_FILE_NAME = "objects_init.xml" ;

FWObjectDatabaseGUI::FWObjectDatabaseGUI() : FWObjectDatabase()
{
    ScratchPad        = new Group();
    ScratchPad->setName("ScratchPad");
}

FWObjectDatabaseGUI::FWObjectDatabaseGUI(const string &filename) throw(FWException)
:FWObjectDatabase()
{
    ScratchPad        = new Group();
    ScratchPad->setName("ScratchPad");
    load(filename);
}

FWObjectDatabaseGUI::~FWObjectDatabaseGUI()
{
}

bool FWObjectDatabaseGUI::saveIfModified() throw(FWException)
{
    MessageDialog::DlgReturnValue action;
    
    action=MessageDialog::Question(
        "Some objects have been modified but not saved.
Do you want to save changes now ?");
    
    switch (action) 
    {
    case MessageDialog::OK_YES: save(); break; 
    case MessageDialog::NO:             break;         
    case MessageDialog::CANCEL:         return(false);
    }
    return(true);
}

void FWObjectDatabaseGUI::load(const string &f) throw(FWException)
{
    if(isDirty(true)) 
    {   // checking all the object recursively
	if (!saveIfModified()) 
            return;  // "Cancel"
    }
    
    if(f=="") 
    {
	FileSel *fs=new FileSel("Select file",
				Preferences::global_prefs->getWdir(),
				"*.xml");
	string df=fs->run();
	delete fs;
	if (df=="") return;
	data_file = df;
    } else
	data_file = f;
    
    if(data_file=="") 
        return;
    
    MessageBoxUpgradePredicate upgrade_predicate;
    FWObjectDatabase::load(f, &upgrade_predicate);

    ScratchPad->clearChildren();
    copyToScratchPad();
    
    if (main_w)
	main_w->showStatusbarMsg(string("Loaded data: ")+data_file);
}

void FWObjectDatabaseGUI::save() throw(FWException)
{
    if (data_file=="") 
    {
	saveAs();
	return;
    }

    saveFile(data_file);
}

void FWObjectDatabaseGUI::saveAs() throw(FWException)
{
    string df;

    FileSel *fs=new FileSel("Select file",
			    Preferences::global_prefs->getWdir(),
			    "*.xml");
    df=fs->run();
    delete fs;
    if (df=="") return;
    data_file=df;

    saveFile(data_file);
}

void FWObjectDatabaseGUI::saveFile(const string &filename) throw(FWException)
{
    FWObjectDatabase::saveFile(filename);
    if(main_w)
	main_w->showStatusbarMsg(string("Saved data as: ")+filename);
}


/*
 *  This method finds a scratch pad copy of object "obj"
 */
FWObject* FWObjectDatabaseGUI::getScratchCopy(FWObject *obj)
{
    if (obj==FWObjectDatabaseGUI::db) return FWObjectDatabaseGUI::ScratchPad;

    FWObject *p=FWObjectDatabaseGUI::ScratchPad->getById( obj->getId(), true );
    return p;
}

void FWObjectDatabaseGUI::copyToScratchPad()
{
    FWObject *p=ScratchPad;    *p=*this;
}

FWObject* FWObjectDatabaseGUI::create(const string& type_name)
{
    FWObject   *nobj=FWObjectDatabase::create(type_name);
    if(nobj==NULL)
        cerr << "unknown node: " << type_name << endl;
    
    return(nobj);
}



