/* 

                          Firewall Builder

                 Copyright (C) 2000 Vadim Kurland

  Author:  Vadim Kurland     vadim@vk.crocodile.org

  $Id: MessageDialog.cc,v 1.19 2001/09/27 02:50:04 vkurland Exp $


  This program is free software which we release under the GNU General Public
  License. You may redistribute and/or modify this program under the terms
  of that license as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  To get a copy of the GNU General Public License, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/



#include <gtk--/main.h>

#include "MessageDialog.hh"
#include "helpers.hh"

#include <gdk_imlib.h>

#include <libgnomeui/gnome-stock.h>
#include "gnome-wrappers/wrappers.hh"


using namespace libfwbuilder;


MessageDialog::MessageDialog(const string& msg,DlgType t)
{
    GdkPixmap *pmap;
    GdkBitmap *mask;
    string     icn;


    running=false;

    result=CANCEL;

    dialog_message->set_text(msg);
    dialog_message->set_justify(GTK_JUSTIFY_LEFT);


/*
 *  button1 : Yes
 *  button2 : No
 *  button3 : OK
 *  button4 : Cancel
 */


    show_all();

    button1->hide();
    button2->hide();
    button3->hide();
    button4->hide();

    switch (t) {
    case QuestionDlg:
	icn = Resources::global_res->getIconPath("Question");
	button1->show();
	button2->show();
	button4->show();
	break;

    case YesNoDlg:
	icn = Resources::global_res->getIconPath("Question");
	button1->show();
	button2->show();
	break;

    case ErrorDlg:
	icn = Resources::global_res->getIconPath("Error");
	button3->show();
	break;

    case WarningDlg:
	icn = Resources::global_res->getIconPath("Warning");
	button3->show();
	break;
    }
    
    if (gdk_imlib_load_file_to_pixmap((char*)(icn.c_str()),&pmap, &mask)){
	pixmap->set(pmap,mask);
	gdk_imlib_free_pixmap(pmap);
    }

    delete_event.connect( SigC::slot(this, &MessageDialog::on_delete) );
}

MessageDialog::DlgReturnValue MessageDialog::run()
{
    Gtk::Main::grab_add(*this);
    running=true;
    Gtk::Main::run();
    running=false;
    Gtk::Main::grab_remove(*this);
    hide();
    return result;
}

gint MessageDialog::on_delete(GdkEventAny* ev)
{
    if (running) {
	Gtk::Main::quit();
	running=false;
    }
    return(true);
}


/*
 *  button1 : Yes
 *  button2 : No
 *  button3 : OK
 *  button4 : Cancel
 */
void MessageDialog::on_button1_clicked()
{   
    result=OK_YES;
    Gtk::Main::quit();
    running=false;
}

void MessageDialog::on_button2_clicked()
{   
    result=NO;
    Gtk::Main::quit();
    running=false;
}

void MessageDialog::on_button3_clicked()
{   
    result=OK_YES;
    Gtk::Main::quit();
    running=false;
}

void MessageDialog::on_button4_clicked()
{   
    result=CANCEL;
    Gtk::Main::quit();
    running=false;
}

void MessageDialog::Error(const string& msg)
{
    MessageDialog *md = new MessageDialog(msg,ErrorDlg);
    md->run();
    delete md;
}

void MessageDialog::Error(const string& msg, const string& errcode)
{
    string m;
    m=msg + "\n" + errcode;
    MessageDialog *md = new MessageDialog(m,ErrorDlg);
    md->run();
    delete md;
}

void MessageDialog::Warning(const string& msg)
{
    MessageDialog *md = new MessageDialog(msg,WarningDlg);
    md->run();
    delete md;
}

MessageDialog::DlgReturnValue  MessageDialog::Question(const string& msg)
{
    DlgReturnValue r;
    MessageDialog *md = new MessageDialog(msg,QuestionDlg);
    r=md->run();
    delete md;
    return r;
}


MessageDialog::DlgReturnValue  MessageDialog::YesNo(const string& msg)
{
    DlgReturnValue r;
    MessageDialog *md = new MessageDialog(msg,YesNoDlg);
    r=md->run();
    delete md;
    return r;
}












