/* 

                          Firewall Builder

                 Copyright (C) 2000 Vadim Kurland

  Author:  Vadim Kurland     vadim@vk.crocodile.org

  $Id: fwbuilder.cc,v 1.58 2001/10/07 07:28:14 vkurland Exp $


  This program is free software which we release under the GNU General Public
  License. You may redistribute and/or modify this program under the terms
  of that license as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  To get a copy of the GNU General Public License, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "config.h"

#ifdef NEED_GNOMESUPPORT_H
#include <gnome-wrappers/gnomemain.hh>
#else
#include <gtk--/main.h>
#endif

#include <glib.h>
#include <gdk_imlib.h>

#include "FWObject.hh"
#include "FWObjectDatabaseGUI.hh"
#include "helpers.hh"
#include "ttips.hh"

#include "main_window.hh"
#include "FWObjectBook.hh"

#include "Resources.hh"
#include "Preferences.hh"
#include "MessageDialog.hh"
#include "XMLTools.hh"

#include "glademm_support.hh"

#include <config.h>

using namespace libfwbuilder;


class StaticInitializer
{
    public:
    
    StaticInitializer()
    {
        /* init threads */
        g_thread_init(NULL);
    }
} init;

static char      *filename   = NULL;
main_window      *main_w     = NULL;

struct poptOption options[] = 
{
    { "file", 'f', POPT_ARG_STRING, &filename, 0,
      "File to load on startup", "filename" },
    POPT_AUTOHELP
    { NULL, 0, 0, NULL, 0 }
};

/**
 *   This handler gets called whenever gtk event loop is stopped via 
 *   Gtk::Main::quit()
 */

gint quit_handler()
{
    if (gtk_main_level()==1) {
// this is the  end of program


    }
    return true;
}


int main(int argc, char **argv)
{   
    gdk_threads_enter();

    /* Init XML parser */
    XMLTools::initXMLTools();


    Gtk::Main m(&argc, &argv);

    gnome_init_with_popt_table("fwbuilder", VERSION, argc, argv, options,0,0 );

/* pretty much everything else in the GUI uses resources, so we start here */
    try  
    {
        new Resources();       // creates singleton Resources::global_res
    } catch(FWException &ex)  
    {
        /*
         *  Error loading resources. MessageDialog needs Resources (for
         *  icons), so we can't use it to show error message.  
         */
	cerr << ex.toString() << endl;
        exit(1);
    }


    try  {
        new Preferences();     // creates singleton Preferences::global_prefs
    } catch(FWException &ex)  {
	MessageDialog::Error(ex.toString());
        exit(1);
    }

   
    main_w = new main_window();

    if ( Preferences::global_prefs->getOptBool(
	     "/FWBuilderPreferences/UI/RememberWindowPositionAndSize") ) {

	main_w->restoreWindowConfiguration();
    } 
    main_w->show_all();

    main_w->restoreWindowPosition();


    new FWObjectClipboard(); // creates singleton 
    new FWObjectDatabaseGUI();  // creates db and scratchpad

    try   {
	if(filename)  {
	    //load file given on command line
	    FWObjectDatabaseGUI::getInstance()->load(filename);
	} else  {
	    //load default file
	    string dbTemplateFile = 
		TEMPLATE_DIR  "/" + FWObjectDatabaseGUI::DB_TEMPLATE_FILE_NAME;
	    FWObjectDatabaseGUI::getInstance()->load(dbTemplateFile);
	    FWObjectDatabaseGUI::getInstance()->setFileName("");
	}
    } catch(FWException &ex)  {
	MessageDialog::Error(ex.toString());
	exit(1);
    }
   
    FWObjectBook *book=(FWObjectBook*)glademm_get_Widget("FWObjectBook");
    assert (book!=NULL);

    book->build();
    main_w->schedule_open_object(FWObjectDatabaseGUI::db->std.rootId);

    InitializeTooltips();

    Gtk::Main::quit.connect( SigC::slot( &quit_handler )  ) ;

    m.run();
    gdk_threads_leave();

    if ( Preferences::global_prefs->getOptBool(
	     "/FWBuilderPreferences/UI/RememberWindowPositionAndSize") ) {

	main_w->saveWindowConfiguration();
	
    }

    return 0;
}









