/* 

                          Firewall Builder

                 Copyright (C) 2000 Vadim Kurland

  Author:  Vadim Kurland     vadim@vk.crocodile.org

  $Id: main_window2.cc,v 1.28 2001/10/07 07:28:14 vkurland Exp $


  This program is free software which we release under the GNU General Public
  License. You may redistribute and/or modify this program under the terms
  of that license as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  To get a copy of the GNU General Public License, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/



#include <iostream>
#include <gtk--/base.h>
#include <gtk--/main.h>
#include <gtk--/tree.h>

#include "glademm_support.hh"

#include "helpers.hh"

#include "main_window.hh"

#include "FWObjectDatabase.hh"
#include "FWObjectDatabaseGUI.hh"
#include "Host.hh"
#include "Firewall.hh"
#include "Network.hh"

#include "Group.hh"
#include "ObjectGroup.hh"
#include "ServiceGroup.hh"

#include "Firewall.hh"
#include "FWOptions.hh"
#include "InterfacePolicy.hh"
#include "Policy.hh"
#include "NAT.hh"

#include "GroupDialog.hh"
#include "BuiltinDialog.hh"
#include "DialogPlugin.hh"
#include "Preferences.hh"

#include "FindDialog.hh"

#include "gen_popup_menu.hh"


#include "MessageDialog.hh"

using namespace libfwbuilder;


bool main_window::safe_to_close_right_pane()
{
  if ( current_right_pane_dialog!=NULL &&
       current_right_pane_dialog->IsDataChanged()) {
    // unsaved changes exist
      
      if ( Preferences::global_prefs->getOptBool("/FWBuilderPreferences/UI/Autosave") ) {
	  Gtk::Widget *w=right_pane_port->get_child();
	  if (w) {
	      BuiltinDialog *dlg = dynamic_cast<BuiltinDialog*>(w);
	      if (dlg)
		  dlg->SaveData();
	  }
	  return true;

      } else {

	  MessageDialog::Warning(
"Data in the current dialog
has been modified. Please
save or discard your changes.");

	  return false;
      }
  }
  return true;
}


void main_window::schedule_open_object(string id)
{
    oo_obj=id;
    Gtk::Main::idle.connect(slot(this,&main_window::open_object_when_idle));
}

gint main_window::open_object_when_idle()
{
    if (oo_obj=="") return false;
    OpenObject(oo_obj);
    oo_obj="";
    return false;
}

void main_window::ClearRightPane(bool logo)
{
    if(right_pane_port)
    {
        Gtk::Widget *w=right_pane_port->get_child();
        if (w!=0) {
            w->hide();
            right_pane_port->remove(); 

	    if ( ! Gtk::Label::isA(w))  delete w;

	    current_right_pane_dialog=NULL;
        }
    }      

    if (logo) 
    {
        /*
         *   TODO:  show logo in the right pane if there is going to be no dialog
         */
    }

    Gtk::Widget *itm;

    itm=find_widget("duplicate",main_menubar1);
    if (itm) itm->set_sensitive(false);

    itm=find_widget("copy",main_menubar1);
    if (itm) itm->set_sensitive(false);

    itm=find_widget("cut",main_menubar1);
    if (itm) itm->set_sensitive(false);

    itm=find_widget("paste",main_menubar1);
    if (itm) itm->set_sensitive(false);
}

bool main_window::OpenObject (string id)
{
    if(!safe_to_close_right_pane())         return false;
    
    ClearRightPane(false);

/*
 *  this automatically switches to the right tree view if necessary
 */
//    if (pane_docked) obook->showObject(id , current_lib );
//    else             obook->clearSelection();

    obook->showObject(id , current_lib );

/* child of right_pane_port is deleted in ClearRightPane */
    BuiltinDialog *dptr = new BuiltinDialog(id);
    
    if(dptr) 
    {
	dptr->LoadData();

	//  right_pane_port->remove(); 
	right_pane_port->add( *dptr );
        
	dptr->show();
	right_pane_port->show();

	current_right_pane_dialog=dptr;
    }
    
    if (dptr && dptr->getDialog()!=NULL)
	dptr->getDialog()->updateMainMenu();

//    FWObject *o=nowShowingObject();

    return true;
}



FWObject*  main_window::newObject(const string& where_id , FWObject *o)
{
    if (o->getName()=="") {  // no name, use type description string instead
	o->setName( Resources::global_res->getObjResourceStr(o,"description") );
    }

    FWObject *s=FWObjectDatabase::db->create( o->getTypeName() );

    FWObjectDatabase::db->addAt(where_id,o);
    *s = *o;
    FWObjectDatabaseGUI::ScratchPad->addAt(where_id,s);

    obook->showPage(0);  // User-defined objects
    obook->insertObject( o );

    return o;
}


FWObject*  main_window::newObject(const string& where_id , const string& type)
{
    FWObject *o=FWObjectDatabase::db->create(type);
    newObject(where_id,o);
    return o;
}


void  main_window::on_new_fw()
{
    if ( safe_to_close_right_pane() ) {
	FWObject *fw=FWObjectDatabase::db->create("Firewall");
	fw->add( new FirewallOptions() );
	fw->add( new Policy() );
	fw->add( new NAT() );
	(Firewall::cast(fw))->setDefaults();
	newObject( FWObjectDatabase::db->std.FirewallsId , fw );
    }
}

void  main_window::on_new_host()
{
    if ( safe_to_close_right_pane() ) 
	newObject( FWObjectDatabase::db->std.HostsId , "Host" );
}

void  main_window::on_new_net()
{
    if ( safe_to_close_right_pane() ) 
	newObject( FWObjectDatabase::db->std.NetworksId , "Network" );
}

void  main_window::on_new_obj_group()
{
    if ( safe_to_close_right_pane() ) 
	newObject( FWObjectDatabase::db->std.ObjectGroupsId,"ObjectGroup");
}

void  main_window::on_new_ip()
{
    if ( safe_to_close_right_pane() ) 
	newObject(FWObjectDatabase::db->std.IPServicesId, "IPService");
}

void  main_window::on_new_icmp()
{
    if ( safe_to_close_right_pane() ) 
	newObject( FWObjectDatabase::db->std.ICMPServicesId, "ICMPService");
}

void  main_window::on_new_tcp()
{
    if ( safe_to_close_right_pane() ) 
	newObject( FWObjectDatabase::db->std.TCPServicesId, "TCPService");
}

void  main_window::on_new_udp()
{
    if ( safe_to_close_right_pane() ) 
	newObject( FWObjectDatabase::db->std.UDPServicesId, "UDPService");
}

void  main_window::on_new_srv_group()
{
    if ( safe_to_close_right_pane() ) 
	newObject( FWObjectDatabase::db->std.ServiceGroupsId, "ServiceGroup");
}

void  main_window::on_new_custom()
{
    if ( safe_to_close_right_pane() ) 
	newObject( FWObjectDatabase::db->std.CustomServicesId,"CustomService");
}

void  main_window::on_new_time()
{
    if ( safe_to_close_right_pane() ) 
	newObject( FWObjectDatabase::db->std.TimeId, "Interval");
}




void main_window::on_cutobj()
{   
    string id=obook->getCurrentSelection();
    FWObject *obj=FWObjectDatabase::db->getById( id, true);
    if ( ! Resources::isSystem(obj) && 
	 ! InterfacePolicy::isA(obj)  && 
	 ! Policy::isA(obj)  && 
	 ! NAT::isA(obj) ) {

	FWObjectClipboard::obj_clipboard->putObject(obj);
	on_delobj();
    }
}

void main_window::on_delobj()
{   
    string id=obook->getCurrentSelection();
    FWObject *obj=FWObjectDatabase::db->getById( id, true);
    if ( ! Resources::isSystem(obj) && 
	 ! InterfacePolicy::isA(obj)  && 
	 ! Policy::isA(obj)  && 
	 ! NAT::isA(obj) ) {

	ClearRightPane(true);

	FWObject *p=obj->getParent();
	obook->removeObject(obj);

	FWObjectDatabase::db->removeAllInstances(obj);
	obj=FWObjectDatabaseGUI::ScratchPad->getById( id , true);
	FWObjectDatabaseGUI::ScratchPad->removeAllInstances(obj);

	OpenObject( p->getId() );
    }
}


void main_window::on_duplicate()
{   
    string id=obook->getCurrentSelection();

    if ( safe_to_close_right_pane() ) {

	FWObject *obj=FWObjectDatabase::db->getById( id , true);

	if (obj) {
	    FWObject *parent=obj->getParent();
	    assert (parent!=NULL);
//          pasteTo(parent,obj);

	    FWObject *no=FWObjectDatabase::db->create(obj->getTypeName());
	    assert(no!=NULL);

	    if (parent->validateChild(no) ) {
		no->duplicate(obj);
		newObject( parent->getId() , no );
	    } else 
		delete no;
	}
    }
}

void main_window::on_copyobj()
{   
    string id=obook->getCurrentSelection();
    if ( safe_to_close_right_pane() ) {
	FWObject *obj=FWObjectDatabase::db->getById( id , true);
	if (obj && ! Resources::isSystem(obj) ) 
	    FWObjectClipboard::obj_clipboard->putObject( id );
    }
}

void  main_window::on_pasteobj()
{
    if ( safe_to_close_right_pane() ) {

	FWObject *obj=FWObjectClipboard::obj_clipboard->getObject();
	if (obj==NULL) return;

	FWObject *cursel=
	    FWObjectDatabase::db->getById( obook->getCurrentSelection() , true);
	if (cursel==NULL) return;

	pasteTo(cursel,obj);
    }
}

void main_window::pasteTo(FWObject *target, FWObject *obj)
{
    if (! Resources::isSystem(target) &&
	( target->getTypeName()==ObjectGroup::TYPENAME ||
	  target->getTypeName()==ServiceGroup::TYPENAME ) ) {

	BuiltinDialog *b=getRightPaneDialog();
	if (b) {
	    GroupDialog   *gd=dynamic_cast<GroupDialog*>(b->getDialog());
	    if (gd) {
		gd->addObject(obj);
	    }
	}
	return;
    }
/*
    FWObject *no=FWObjectDatabase::db->create(obj->getTypeName());
    assert(no!=NULL);

    if (target->validateChild(no) ) {
	no->duplicate(obj);
	newObject( target->getId() , no );
    } else 
	delete no;
*/
}

void main_window::on_sort()
{
    FWObject *o;
    string id=obook->getCurrentSelection();

    o=FWObjectDatabase::db->getById( id , true ) ;
    assert(o!=NULL);
    o->sortChildren();

    o=FWObjectDatabaseGUI::ScratchPad->getById( id , true ) ;
    assert(o!=NULL);
    o->sortChildren();

//    obook->scheduleRebuildAndShow(id);
    obook->sortSubtree(id);
    OpenObject(id);
}

void main_window::on_find()
{
    if (glademm_get_Widget("FindDialog")==NULL)
	manage(new FindDialog());
}



